Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type CheckpointWriter ¶
type CheckpointWriter interface {
// StoreCheckpoints writes at least minCheckpoints if there are more checkpoints to write.
// Checkpoints are written until ctx permits or all checkpoints are written.
StoreCheckpoints(ctx context.Context, now time.Time, minCheckpoints int) error
}
CheckpointWriter persistently stores aggregated historical usage of containers controlled by VPA objects. This state can be restored to initialize the model after restart.
func NewCheckpointWriter ¶
func NewCheckpointWriter(cluster *model.ClusterState, vpaCheckpointClient vpa_api.VerticalPodAutoscalerCheckpointsGetter) CheckpointWriter
Click to show internal directories.
Click to hide internal directories.