Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type BasicContainerSpec ¶
type BasicContainerSpec struct {
// ID identifies the container within a cluster.
ID model.ContainerID
// Name of the image running within the container.
Image string
// Currently requested resources for this container.
Request model.Resources
}
BasicContainerSpec contains basic information defining a container.
type BasicPodSpec ¶
type BasicPodSpec struct {
// ID identifies a pod within a cluster.
ID model.PodID
// Labels of the pod. It is used to match pods with certain VPA opjects.
PodLabels map[string]string
// List of containers within this pod.
Containers []BasicContainerSpec
// PodPhase describing current life cycle phase of the Pod.
Phase v1.PodPhase
}
BasicPodSpec contains basic information defining a pod and its containers.
type SpecClient ¶
type SpecClient interface {
// Returns BasicPodSpec for each pod in the cluster
GetPodSpecs() ([]*BasicPodSpec, error)
}
SpecClient provides information about pods and containers Specification
func NewSpecClient ¶
func NewSpecClient(podLister v1lister.PodLister) SpecClient
Click to show internal directories.
Click to hide internal directories.